      subroutine  eigen_sx( n, nvec, a, lda, w, z, ldz,
     &     m_forward, m_backward, mode )
!$    use OMP_LIB
      use MPI
      use eigen_devel
      use eigen_libs, only : eigen_get_matdims
      implicit NONE
!-
      integer, intent(IN)    :: n, nvec, lda, ldz
      real(8), intent(INOUT) :: a(lda,*)
      real(8), intent(OUT)   :: w(*), z(ldz,*)
      integer, intent(IN), optional :: m_forward, m_backward
      character, intent(IN), optional :: mode
!-
      real(8), pointer       :: d(:), e(:), e2(:)
!-
      integer                :: mf, mb
      character              :: mode_
!-
      real(8)                :: SIGMA
      integer                :: nm, ny, nb, nme
      integer                :: my_rank, world_size, ierr
      integer                :: INFO
      logical                :: flag
!-
      real(8)                :: hs0, hs1, s0, s1
      real(8)                :: ret, ret_1, ret_2, ret_3
!-
!-------------------------------------------------------------------
!-
      call eigen_get_initialized( flag )
      if ( .NOT. flag ) then
         return
      end if
      if ( TRD_COMM_WORLD == MPI_COMM_NULL ) then
         return
      end if
!-
!-------------------------------------------------------------------
!-
      if ( n <= 0 ) then
         print*,"Warining: Negative dimesion is invalid!"
         return
      end if

      call eigen_get_matdims( n, nm, ny )
      if ( nm <= 0 .OR. ny <= 0 ) then
         print*,"Warining: Problem size is too large for"
     &        //  " 32bit fortarn integer binary."
         return
      end if
!-
!-------------------------------------------------------------------
!-
      if ( PRESENT( mode ) ) then
         mode_ = mode
      else
         mode_ = 'A'
      end if

      if ( PRESENT( m_forward ) ) then
         mf = m_forward
      else
         mf = 48
      end if
      if ( mode_ /= 'N' ) then
         if ( PRESENT( m_backward ) ) then
            mb = m_backward
         else
            mb = 128
         end if
      end if
!-
!-------------------------------------------------------------------
!-
      hs0 = eigen_get_wtime()
      ret_1 = ZERO; ret_2 = ZERO; ret_3 = ZERO
!-
!-------------------------------------------------------------------
!-
      nme = ((n-1)/4+1)*4
      allocate( d(1:n), e(1:nme*2), e2(1:nme*2) )
!-
      world_size = TRD_nnod
      my_rank    = TRD_inod-1
!-
!-------------------------------------------------------------------
!-
      SIGMA = ONE
!     if ( PRESENT( mode ) ) then
!     if ( mode == 'X' ) then
      call eigen_scaling( n, a(1,1), lda, SIGMA )
!     end if
!     end if
!-
!-------------------------------------------------------------------
!-
#if TIMER_PRINT
      s0 = eigen_get_wtime( )
#endif

      call eigen_prd( n, a(1,1), lda, d(1), e(1), nme, mf )
      ret_1 = dble(n)**3*4/3

#if TIMER_PRINT
      s1 = eigen_get_wtime( )
      if ( my_rank == 0 ) then
         print 10000,"TRD-BLK ", n, s1-s0, 1D-9*ret_1/(s1-s0), "GFLOPS"
      end if
      call flush(6)
#endif
!-
!-------------------------------------------------------------------
!-
#if TIMER_PRINT
      s0 = eigen_get_wtime( )
#endif

      if ( mode == 'A' .OR. mode == 'X' ) then
         e2(0*nme+1:0*nme+N-1) = e(0*nme+2:0*nme+N)
         e2(0*nme+N) = 0
         e2(1*nme+1:1*nme+N-2) = e(1*nme+3:1*nme+N)
         e2(1*nme+N-1) = 0
         e2(1*nme+N) = 0
         w(1:n)=d(1:n)
         call eigen_dcx( n, w, e2, nme, z, ldz, INFO, ret_2 )
         if ( mode == 'X' ) then
            call eigen_bisect2( d, e, e(1+nme), w, n, 1 )
         end if
      end if
      if ( mode == 'N' ) then
         call eigen_bisect2( d, e, e(1+nme), w, n, 0 )
      end if

#if TIMER_PRINT
      s1 = eigen_get_wtime( )
      if ( my_rank == 0 ) then
         print 10000,"D&C     ", n, s1-s0, 1D-9*ret_2/(s1-s0), "GFLOPS"
      end if
      call flush(6)
#endif
      if ( mode_ == 'N' ) then
         goto 99999
      end if
!-
!-------------------------------------------------------------------
!-
#if TIMER_PRINT
      s0 = eigen_get_wtime( )
#endif

      nb = MBAND
      call eigen_trbakwyx( n, nvec, a(1,1), lda, z(1,1), ldz,
     &     e(1+nme), mb, nb )
      ret_3 = 2*dble(nvec)*dble(n)**2

#if TIMER_PRINT
      s1 = eigen_get_wtime( )
      if ( my_rank == 0 ) then
         print 10000,"TRDBAK  ", n, s1-s0, 1D-9*ret_3/(s1-s0), "GFLOPS"
      end if
      call flush(6)
#endif
!-
!-------------------------------------------------------------------
!-
      IF ( SIGMA /= ONE ) THEN
         if ( my_rank == 0 ) then
            print*,"Scaling done ",sigma
         end if
         SIGMA = ONE / SIGMA
         call dscal( n, w, SIGMA, 1 )
      END IF
!-
!-------------------------------------------------------------------
!-
#if TIMER_PRINT
      hs1 = eigen_get_wtime ( )
      ret = ret_1 + ret_2 + ret_3
      if ( my_rank == 0 ) then
         print*,"Total(eigen_sx)",
     &        hs1-hs0, 1D-9*ret/(hs1-hs0), "GFLOPS"
      end if
      call flush(6)
#endif
!-
!-------------------------------------------------------------------
!-
99999 continue
!-
!-------------------------------------------------------------------
!-
      hs1 = eigen_get_wtime ( )
      ret = ret_1 + ret_2 + ret_3
      if ( ret_2 == 0 ) ret = -ret
      a(1, 1) = ret
      a(2, 1) = hs1-hs0
#if TIMER_PRINT
      a(3, 1) = comm_time_reduction
     &     + comm_time_dc
     &     + comm_time_backtrafo
#else
      a(3, 1) = -1D0
#endif
!-
!-------------------------------------------------------------------
!-
      deallocate( d, e, e2 )
!-
!-------------------------------------------------------------------
!-
10000 format ( X, A8, I8, E25.16e2, E25.16e2, X, A )
!-
!-------------------------------------------------------------------
!-
      end subroutine  eigen_sx

