!======================================================================c
      subroutine  eigen_2update(
     &     ar, nma,
     &     ur, uyr, vr, vyr, nmv,
     &     m_size, i_base)
!======================================================================c
!$    use OMP_LIB
      use eigen_devel
      use eigen_libs
      implicit NONE
!     
      real(8), intent(INOUT) :: ar (nma, *)
      real(8), intent(IN)    :: ur (nmv, *)
      real(8), intent(IN)    :: uyr(nmv, *)
      real(8), intent(IN)    :: vr (nmv, *)
      real(8), intent(IN)    :: vyr(nmv, *)
      integer, intent(IN)    :: nma, nmv, m_size, i_base
!     
      integer ::  i, j, k, k0, k1, k2, L, m, n

      integer ::  i_1,i_2,i_3,i_4
      integer ::  j_1,j_2,j_3,j_4

      integer ::  ii_1,ii_2,ii_3,ii_4
      integer ::  jj_1,jj_2,jj_3,jj_4

      integer ::  blk_size1, blk_size2
      integer ::  ii_step
!     
      integer ::  local_rank, local_size
!     
#if __FUJITSU
      integer            :: BLAS_CHUNK_N
      integer            :: BLAS_CHUNK_M
#else
      integer, parameter :: BLAS_CHUNK_N = 8*4
      integer, parameter :: BLAS_CHUNK_M = 96
#endif
!     
      intrinsic :: MIN, MAX
      external  :: DGEMM
!     
!======================================================================c
!     
      if ( i_base <= 0 ) return
!     
!======================================================================c
!     
      local_rank = 0
      local_size = 1
!$    local_rank = omp_get_thread_num()
!$    local_size = omp_get_num_threads()
!     
      m  = m_size               ! preserve the argument variable 
      n  = eigen_translate_g2l  (i_base, y_nnod, y_inod) ! local matrix 
!     ! size

!     12 + 8 + 4 = 24 = 8*3
#if __FUJITSU
!     BLAS_CHUNK_M = MAX( (n-1)/ 3+1, 32 )
!     BLAS_CHUNK_N = MAX( (n-1)/12+1,  8 )
!     BLAS_CHUNK_M = ((BLAS_CHUNK_M-1)/4+1)*4
!     BLAS_CHUNK_N = ((BLAS_CHUNK_N-1)/2+1)*2
      BLAS_CHUNK_M = 256*3
      BLAS_CHUNK_N = 7*8
#endif

      jj_2 = 1                  ! beggining of loop
      jj_3 = eigen_loop_end  (i_base, x_nnod, x_inod) ! end of loop

      ii_step = 0
      do j_1 = jj_2, jj_3, BLAS_CHUNK_M
         j_4 = MIN(j_1+BLAS_CHUNK_M-1, jj_3) ! [j_1:j_4] available on
!     ! this iteration 

         k1   = eigen_translate_l2g(j_1, x_nnod, x_inod) ! translation
!     ! to the global index
         ii_2 = eigen_loop_start(k1,     y_nnod, y_inod) ! beggining of 
!     ! loop
         ii_2 = MAX(1, ii_2)    !  ** should be .GE. 1
         ii_3 = eigen_loop_end  (i_base, y_nnod, y_inod) ! end of loop

         do i_1 = ii_2, ii_3, BLAS_CHUNK_N
            i_4 = MIN(i_1+BLAS_CHUNK_N-1, ii_3) ! [i_1:i_4] available on
!     ! this iteration

            i_2 = i_1; i_3 = i_4

            k2  = eigen_translate_l2g(i_4, y_nnod, y_inod) ! translation
!     ! to the global index
            j_2 = j_1
            j_2 = MAX(1, j_2)   !  ** should be .GE. 1
            j_3 = eigen_loop_end  (k2, x_nnod, x_inod) ! end of loop
            j_3 = MIN(j_4, j_3) !  ** should be .LE. j_4

            blk_size1 = j_3-j_2+1
            blk_size2 = i_3-i_2+1

            if ( blk_size1 > 0 .AND. blk_size2 > 0 ) then

               if ( MOD(ii_step, local_size) == local_rank ) then

                  call DGEMM('N','T',
     &                 blk_size1, blk_size2, m,
     &                 MONE, ur (j_1, 1),   nmv,
     &                 vyr(i_1, 1),   nmv,
     &                 ONE,  ar (j_1, i_1), nma)

                  call DGEMM('N','T',
     &                 blk_size1, blk_size2, m,
     &                 MONE, vr (j_1, 1),   nmv,
     &                 uyr(i_1, 1),   nmv,
     &                 ONE,  ar (j_1, i_1), nma)

               end if

               ii_step = ii_step+1

            end if

         end do                 ! i_1

      end do                    ! j_1
!-
      return
      end subroutine  eigen_2update
!======================================================================c

